@@ -120,6 +120,8 @@ class Scheduler:
         By default a full parse is executed, use this flag to suppress.
     frontend : :any:`Frontend`, optional
         Frontend to use for full parse of source files (default :any:`FP`).
+    output_dir : str or path
+        Directory for the output to be written to
     """
 
     # TODO: Should be user-definable!
@@ -127,14 +129,14 @@ class Scheduler:
 
     def __init__(self, paths, config=None, seed_routines=None, preprocess=False,
                  includes=None, defines=None, definitions=None, xmods=None,
-                 omni_includes=None, full_parse=True, frontend=FP):
+                 omni_includes=None, full_parse=True, frontend=FP, output_dir=None):
         # Derive config from file or dict
         if isinstance(config, SchedulerConfig):
             self.config = config
         elif isinstance(config, (str, Path)):
             self.config = SchedulerConfig.from_file(config)
         else:
-            self.config = SchedulerConfig.from_dict(config)
+            self.config = SchedulerConfig.from_dict(config or {})
 
         self.full_parse = full_parse
 
@@ -153,7 +155,8 @@ class Scheduler:
             'defines': defines,
             'xmods': xmods,
             'omni_includes': omni_includes,
-            'frontend': frontend
+            'frontend': frontend,
+            'output_dir': output_dir
         }
 
         # Internal data structures to store the callgraph
@@ -283,7 +286,7 @@ class Scheduler:
         # Re-build the SGraph after parsing to pick up all new connections
         self._sgraph = SGraph.from_seed(self.seeds, self.item_factory, self.config)
 
-    @Timer(logger=info, text='[Loki::Scheduler] Enriched call tree in {:.2f}s')
+    @Timer(logger=perf, text='[Loki::Scheduler] Enriched call tree in {:.2f}s')
     def _enrich(self):
         """
         For items that have a specific enrichment list provided as part of their
@@ -495,7 +498,7 @@ class Scheduler:
                     _item.scope_ir, role=_item.role, mode=_item.mode,
                     item=_item, targets=_item.targets, items=_get_definition_items(_item, sgraph_items),
                     successors=graph.successors(_item, item_filter=item_filter),
-                    depths=graph.depths
+                    depths=graph.depths, build_args=self.build_args
                 )
 
         if transformation.renames_items:
