@@ -414,21 +414,23 @@ end subroutine rick
 """
     source = Sourcefile.from_source(fcode)
     source.path = Path('rick.F90')
-    item = ProcedureItem(name='#rick', source=source)
-
-    # Test default file writes
-    ricks_path = tmp_path/'rick.loki.F90'
-    if ricks_path.exists():
-        ricks_path.unlink()
-    FileWriteTransformation(builddir=tmp_path).apply(source=source, item=item)
-    assert ricks_path.exists()
-    ricks_path.unlink()
+    item = ProcedureItem(name='#rick', source=source, config={'mode': 'roll'})
 
     # Test mode and suffix overrides
     ricks_path = tmp_path/'rick.roll.java'
     if ricks_path.exists():
         ricks_path.unlink()
-    FileWriteTransformation(builddir=tmp_path, mode='roll', suffix='.java').apply(source=source, item=item)
+    FileWriteTransformation(suffix='.java').apply(source=source, item=item,
+                                                  build_args={'output_dir': tmp_path})
+    assert ricks_path.exists()
+    ricks_path.unlink()
+
+    item = ProcedureItem(name='#rick', source=source)
+    # Test default file writes
+    ricks_path = tmp_path/'rick.loki.F90'
+    if ricks_path.exists():
+        ricks_path.unlink()
+    FileWriteTransformation().apply(source=source, item=item, build_args={'output_dir': tmp_path})
     assert ricks_path.exists()
     ricks_path.unlink()
 
@@ -436,13 +438,13 @@ end subroutine rick
     ricks_path = tmp_path/'rick.loki.F90'
     if ricks_path.exists():
         ricks_path.unlink()
-    FileWriteTransformation(builddir=tmp_path).apply(source=source, items=(item,))
+    FileWriteTransformation().apply(source=source, items=(item,), build_args={'output_dir': tmp_path})
     assert ricks_path.exists()
     ricks_path.unlink()
 
     # Check error behaviour if no item provided
     with pytest.raises(ValueError):
-        FileWriteTransformation(builddir=tmp_path).apply(source=source)
+        FileWriteTransformation().apply(source=source)
 
 
 def test_transformation_pipeline_simple():
