@@ -1236,6 +1236,8 @@ class LiteralList(pmbl.AlgebraicLeaf):
     A list of constant literals, e.g., as used in Array Initialization Lists.
     """
 
+    init_arg_names = ('values', 'dtype')
+
     def __init__(self, values, dtype=None, **kwargs):
         self.elements = values
         self.dtype = dtype
@@ -1395,6 +1397,20 @@ class InlineCall(pmbl.CallWithKwargs):
         kwargs = ((r_args[kw], arg) for kw, arg in as_tuple(self.kwarguments))
         return chain(args, kwargs)
 
+    @property
+    def arg_map(self):
+        """
+        A full map of all qualified argument matches from arguments
+        and keyword arguments.
+
+        Returns
+        -------
+        dict
+            An dictionary that mapping ``arg name: call arg`` for
+            all positional and then keyword arguments.
+        """
+        return dict(self.arg_iter())
+
     def clone(self, **kwargs):
         """
         Replicate the object with the provided overrides.
@@ -1410,17 +1426,26 @@ class Cast(pmbl.Call):
     Internal representation of a data type cast.
     """
 
+    init_arg_names = ('name', 'expression', 'kind')
+
     def __init__(self, name, expression, kind=None, **kwargs):
         assert kind is None or isinstance(kind, pmbl.Expression)
         self.kind = kind
         super().__init__(pmbl.make_variable(name), as_tuple(expression), **kwargs)
 
+    def __getinitargs__(self):
+        return (self.name, self.expression, self.kind)
+
     mapper_method = intern('map_cast')
 
     @property
     def name(self):
         return self.function.name
 
+    @property
+    def expression(self):
+        return self.parameters
+
 
 class Range(StrCompareMixin, pmbl.Slice):
     """
