@@ -12,7 +12,6 @@ import pytest
 import numpy as np
 
 import pymbolic.primitives as pmbl
-import pymbolic.mapper as pmbl_mapper
 
 from loki import (
     Sourcefile, Subroutine, Module, Scope, BasicType,
@@ -20,14 +19,14 @@ from loki import (
 )
 from loki.backend import cgen, fgen
 from loki.build import jit_compile, clean_test
-from loki.expression import (
-    symbols as sym, FindVariables, FindExpressions, FindInlineCalls,
-    SubstituteExpressions, AttachScopesMapper, parse_expr
-)
+from loki.expression import symbols as sym, parse_expr, AttachScopesMapper
 from loki.frontend import (
-    available_frontends, OFP, OMNI, FP, HAVE_FP, parse_fparser_expression
+    available_frontends, OMNI, FP, HAVE_FP, parse_fparser_expression
+)
+from loki.ir import (
+    nodes as ir, FindNodes, FindVariables, FindExpressions,
+    FindInlineCalls, SubstituteExpressions
 )
-from loki.ir import nodes as ir, FindNodes
 from loki.tools import (
     filehash, stdchannel_redirected, stdchannel_is_captured
 )
@@ -208,9 +207,7 @@ end subroutine boz_literals
         assert stmts[5].rhs.parameters[0].value == 'z"babe"'
 
 
-@pytest.mark.parametrize('frontend', available_frontends(
-    skip={OFP: "Not implemented because too stupid in OFP parse tree"})
-)
+@pytest.mark.parametrize('frontend', available_frontends())
 def test_complex_literals(tmp_path, frontend):
     """
     Test complex literal values.
@@ -310,9 +307,7 @@ end subroutine logical_array
     clean_test(filepath)
 
 
-@pytest.mark.parametrize('frontend', available_frontends(
-    xfail=[(OFP, 'Not implemented')]
-))
+@pytest.mark.parametrize('frontend', available_frontends())
 def test_array_constructor(tmp_path, frontend):
     """
     Test various array constructor formats
@@ -796,9 +791,7 @@ end subroutine expression_masked_statements
     clean_test(filepath)
 
 
-@pytest.mark.parametrize('frontend', available_frontends(xfail=[
-    (OFP, 'Current implementation does not handle nested where constructs')
-]))
+@pytest.mark.parametrize('frontend', available_frontends())
 def test_masked_statements_nested(tmp_path, frontend):
     """
     Nested masked statements (WHERE(...) ... [ELSEWHERE ...] ENDWHERE)
@@ -1607,7 +1600,7 @@ end module typebound_resolution_type_info_mod
 ))
 def test_stmt_func_heuristic(frontend, tmp_path):
     """
-    Our Fparser/OFP translation has a heuristic to detect statement function declarations,
+    Our Fparser translation has a heuristic to detect statement function declarations,
     but that falsely misinterpreted some assignments as statement functions due to
     missing shape information (reported in #326)
     """
