@@ -11,6 +11,7 @@ Control flow node classes for
 :ref:`internal_representation:Control flow tree`
 """
 
+from abc import abstractmethod
 from collections import OrderedDict
 from dataclasses import dataclass
 from functools import partial
@@ -22,10 +23,14 @@ from pymbolic.primitives import Expression
 from pydantic.dataclasses import dataclass as dataclass_validated
 from pydantic import model_validator
 
+from loki.expression import (
+    symbols as sym, Variable, parse_expr, AttachScopesMapper,
+    ExpressionDimensionsMapper
+)
+from loki.frontend.source import Source
 from loki.scope import Scope
 from loki.tools import flatten, as_tuple, is_iterable, truncate_string, CaseInsensitiveDict
 from loki.types import DataType, BasicType, DerivedType, SymbolAttributes
-from loki.frontend.source import Source
 
 
 __all__ = [
@@ -329,6 +334,80 @@ class ScopedNode(Scope):
         symbol_attrs = s.pop('symbol_attrs', None)
         self._update(**s, symbol_attrs=symbol_attrs, rescope_symbols=True)
 
+    @property
+    @abstractmethod
+    def variables(self):
+        """
+        Return the variables defined in this :any:`ScopedNode`.
+        """
+
+    @property
+    def variable_map(self):
+        """
+        Map of variable names to :any:`Variable` objects
+        """
+        return CaseInsensitiveDict((v.name, v) for v in self.variables)
+
+    def get_symbol(self, name):
+        """
+        Returns the symbol for a given name as defined in its declaration.
+
+        The returned symbol might include dimension symbols if it was
+        declared as an array.
+
+        Parameters
+        ----------
+        name : str
+            Base name of the symbol to be retrieved
+        """
+        return self.get_symbol_scope(name).variable_map.get(name)
+
+    def Variable(self, **kwargs):
+        """
+        Factory method for :any:`TypedSymbol` or :any:`MetaSymbol` classes.
+
+        This invokes the :any:`Variable` with this node as the scope.
+
+        Parameters
+        ----------
+        name : str
+            The name of the variable.
+        type : optional
+            The type of that symbol. Defaults to :any:`BasicType.DEFERRED`.
+        parent : :any:`Scalar` or :any:`Array`, optional
+            The derived type variable this variable belongs to.
+        dimensions : :any:`ArraySubscript`, optional
+            The array subscript expression.
+        """
+        kwargs['scope'] = self
+        return Variable(**kwargs)
+
+    def parse_expr(self, expr_str, strict=False, evaluate=False, context=None):
+        """
+        Uses :meth:`parse_expr` to convert expression(s) represented
+        in a string to Loki expression(s)/IR.
+
+        Parameters
+        ----------
+        expr_str : str
+            The expression as a string
+        strict : bool, optional
+            Whether to raise exception for unknown variables/symbols when
+            evaluating an expression (default: `False`)
+        evaluate : bool, optional
+            Whether to evaluate the expression or not (default: `False`)
+        context : dict, optional
+            Symbol context, defining variables/symbols/procedures to help/support
+            evaluating an expression
+
+        Returns
+        -------
+        :any:`Expression`
+            The expression tree corresponding to the expression
+        """
+        return parse_expr(expr_str, scope=self, strict=strict, evaluate=evaluate, context=context)
+
+
 # Intermediate node types
 
 
@@ -425,12 +504,49 @@ class Associate(ScopedNode, Section, _AssociateBase):  # pylint: disable=too-man
         """
         An :any:`collections.OrderedDict` of associated expressions.
         """
-        return OrderedDict(self.associations)
+        return CaseInsensitiveDict((k, v) for k, v in self.associations)
+
+    @property
+    def inverse_map(self):
+        """
+        An :any:`collections.OrderedDict` of associated expressions.
+        """
+        return CaseInsensitiveDict((v, k) for k, v in self.associations)
 
     @property
     def variables(self):
         return tuple(v for _, v in self.associations)
 
+    def _derive_local_symbol_types(self, parent_scope):
+        """ Derive the types of locally defined symbols from their associations. """
+
+        rescoped_associations = ()
+        for expr, name in self.associations:
+            # Put symbols in associated expression into the right scope
+            expr = AttachScopesMapper()(expr, scope=parent_scope)
+
+            # Determine type of new names
+            if isinstance(expr, (sym.TypedSymbol, sym.MetaSymbol)):
+                # Use the type of the associated variable
+                _type = expr.type.clone(parent=None)
+                if isinstance(expr, sym.Array) and expr.dimensions is not None:
+                    shape = ExpressionDimensionsMapper()(expr)
+                    if shape == (sym.IntLiteral(1),):
+                        # For a scalar expression, we remove the shape
+                        shape = None
+                    _type = _type.clone(shape=shape)
+            else:
+                # TODO: Handle data type and shape of complex expressions
+                shape = ExpressionDimensionsMapper()(expr)
+                if shape == (sym.IntLiteral(1),):
+                    # For a scalar expression, we remove the shape
+                    shape = None
+                _type = SymbolAttributes(BasicType.DEFERRED, shape=shape)
+            name = name.clone(scope=self, type=_type)
+            rescoped_associations += ((expr, name),)
+
+        self._update(associations=rescoped_associations)
+
     def __repr__(self):
         if self.associations:
             associations = ', '.join(f'{str(var)}={str(expr)}'
@@ -628,6 +744,16 @@ class Conditional(InternalNode, _ConditionalBase):
             return f'Conditional:: {self.name}'
         return 'Conditional::'
 
+    @property
+    def else_bodies(self):
+        """
+        Return all nested node tuples in the ``ELSEIF``/``ELSE`` part
+        of the conditional chain.
+        """
+        if self.has_elseif:
+            return (self.else_body[0].body,) + self.else_body[0].else_bodies
+        return (self.else_body,) if self.else_body else ()
+
 
 @dataclass_strict(frozen=True)
 class _PragmaRegionBase():
@@ -872,6 +998,23 @@ class CallStatement(LeafNode, _CallStatementBase):
 
     _traversable = ['name', 'arguments', 'kwarguments']
 
+    @model_validator(mode='before')
+    @classmethod
+    def pre_init(cls, values):
+        # Ensure non-nested tuples for arguments
+        if 'arguments' in values.kwargs:
+            values.kwargs['arguments'] = _sanitize_tuple(values.kwargs['arguments'])
+        else:
+            values.kwargs['arguments'] = ()
+        # Ensure two-level nested tuples for kwarguments
+        if 'kwarguments' in values.kwargs:
+            kwarguments = as_tuple(values.kwargs['kwarguments'])
+            kwarguments = tuple(_sanitize_tuple(pair) for pair in kwarguments)
+            values.kwargs['kwarguments'] = kwarguments
+        else:
+            values.kwargs['kwarguments'] = ()
+        return values
+
     def __post_init__(self):
         super().__post_init__()
         assert isinstance(self.arguments, tuple)
@@ -1579,10 +1722,6 @@ class TypeDef(ScopedNode, InternalNode, _TypeDefBase):
     def variables(self):
         return tuple(flatten([decl.symbols for decl in self.declarations]))
 
-    @property
-    def variable_map(self):
-        return CaseInsensitiveDict((s.name, s) for s in self.variables)
-
     @property
     def imported_symbols(self):
         """
