@@ -11,9 +11,9 @@ import numpy as np
 
 from loki import Module, Subroutine, fgen
 from loki.build import jit_compile, jit_compile_lib, clean_test, Builder, Obj
-from loki.expression import symbols as sym, FindVariables
+from loki.expression import symbols as sym
 from loki.frontend import available_frontends, OMNI
-from loki.ir import FindNodes, CallStatement, Loop
+from loki.ir import FindNodes, CallStatement, Loop, FindVariables, Assignment
 
 from loki.transformations.array_indexing import (
     promote_variables, demote_variables, invert_array_indices,
@@ -1062,38 +1062,164 @@ end subroutine transform_resolve_vector_notation
     assert np.all(ret1 == 11)
     assert np.all(ret2 == 42)
 
+
+@pytest.mark.parametrize('frontend', available_frontends())
+def test_transform_resolve_vector_notation_common_loops(tmp_path, frontend):
+    """
+    Apply and test resolve vector notation utility with already
+    available/appropriate loops.
+    """
+    fcode = """
+subroutine transform_resolve_vector_notation_common_loops(scalar, vector, matrix, n, m, l)
+  implicit none
+  integer, intent(in) :: n, m, l
+  integer, intent(inout) :: scalar, vector(n), matrix(l, n)
+  integer :: tmp_scalar, tmp_vector(n, m), tmp_matrix(l, m, n), tmp_dummy(n, 0:4)
+  integer :: jl, jk, jm
+
+  tmp_dummy(:,:) = 0
+  tmp_vector(:, 1) = tmp_dummy(:, 1)
+  tmp_vector(:, :) = 0
+  tmp_matrix(:, :, :) = 0
+  matrix(:, :) = 0
+
+  do jl=1,n
+    do jm=1,m
+      tmp_vector(jl, jm) = scalar + jl
+    end do
+  end do
+
+  do jm=1,m
+    do jl=1,n
+      scalar = jl
+      vector(jl) = tmp_vector(jl, jm) + tmp_vector(jl, jm)
+
+      do jk=1,l
+        tmp_matrix(jk, jm, jl) = vector(jl) + jk
+      end do
+    end do
+  end do
+
+
+  do jk=1,l
+    matrix(jk, :) = 0
+    do jm=1,m
+      do jl=1,n
+        matrix(jk, jl) = tmp_matrix(jk, jm, jl)
+      end do
+    end do
+  end do
+
+end subroutine transform_resolve_vector_notation_common_loops
+    """.strip()
+    routine = Subroutine.from_source(fcode, frontend=frontend)
+    # Test the original implementation
+    filepath = tmp_path/(f'{routine.name}_{frontend}.f90')
+    function = jit_compile(routine, filepath=filepath, objname=routine.name)
+
+    n = 3
+    m = 2
+    l = 3
+    scalar = np.zeros(shape=(1,), order='F', dtype=np.int32)
+    vector = np.zeros(shape=(n,), order='F', dtype=np.int32)
+    matrix = np.zeros(shape=(n, n), order='F', dtype=np.int32)
+    function(scalar, vector, matrix, n, m, l)
+
+    assert all(scalar == 3)
+    assert np.all(vector == np.arange(1, n + 1)*2)
+    assert np.all(matrix == np.sum(np.mgrid[1:4,2:8:2], axis=0))
+
+    resolve_vector_notation(routine)
+
+    loops = FindNodes(Loop).visit(routine.body)
+    arrays = [var for var in FindVariables(unique=False).visit(routine.body) if isinstance(var, sym.Array)]
+
+    assert len(loops) == 19
+    assert loops[0].variable == 'i_tmp_dummy_1' and loops[0].bounds.children == (0, 4, None)
+    assert loops[1].variable == 'jl' and loops[1].bounds.children == (1, 'n', 1)
+    assert loops[2].variable == 'jl' and loops[2].bounds.children == (1, 'n', 1)
+    assert loops[3].variable == 'jm' and loops[3].bounds.children == (1, 'm', 1)
+    assert loops[4].variable == 'jl' and loops[4].bounds.children == (1, 'n', 1)
+    assert loops[5].variable == 'jl' and loops[5].bounds.children == (1, 'n', 1)
+    assert loops[6].variable == 'jm' and loops[6].bounds.children == (1, 'm', 1)
+    assert loops[7].variable == 'jk' and loops[7].bounds.children == (1, 'l', 1)
+    assert loops[8].variable == 'jl' and loops[8].bounds.children == (1, 'n', 1)
+    assert loops[9].variable == 'jk' and loops[9].bounds.children == (1, 'l', 1)
+    assert loops[10].variable == 'jl' and loops[10].bounds.children == (1, 'n', None)
+    assert loops[11].variable == 'jm' and loops[11].bounds.children == (1, 'm', None)
+    assert loops[12].variable == 'jm' and loops[12].bounds.children == (1, 'm', None)
+    assert loops[13].variable == 'jl' and loops[13].bounds.children == (1, 'n', None)
+    assert loops[14].variable == 'jk' and loops[14].bounds.children == (1, 'l', None)
+    assert loops[15].variable == 'jk' and loops[15].bounds.children == (1, 'l', None)
+    assert loops[16].variable == 'jl' and loops[16].bounds.children == (1, 'n', 1)
+    assert loops[17].variable == 'jm' and loops[17].bounds.children == (1, 'm', None)
+    assert loops[18].variable == 'jl' and loops[18].bounds.children == (1, 'n', None)
+
+    assert len(arrays) == 15
+    assert arrays[0].name.lower() == 'tmp_dummy' and arrays[0].dimensions == ('jl', 'i_tmp_dummy_1')
+    assert arrays[1].name.lower() == 'tmp_vector' and arrays[1].dimensions == ('jl', 1)
+    assert arrays[2].name.lower() == 'tmp_dummy' and arrays[2].dimensions == ('jl', 1)
+    assert arrays[3].name.lower() == 'tmp_vector' and arrays[3].dimensions == ('jl', 'jm')
+    assert arrays[4].name.lower() == 'tmp_matrix' and arrays[4].dimensions == ('jk', 'jm', 'jl')
+
+    # Test promoted routine
+    resolved_filepath = tmp_path/(f'{routine.name}_resolved_{frontend}.f90')
+    resolved_function = jit_compile(routine, filepath=resolved_filepath, objname=routine.name)
+
+    n = 3
+    m = 2
+    l = 3
+    scalar = np.zeros(shape=(1,), order='F', dtype=np.int32)
+    vector = np.zeros(shape=(n,), order='F', dtype=np.int32)
+    matrix = np.zeros(shape=(n, n), order='F', dtype=np.int32)
+    resolved_function(scalar, vector, matrix, n, m, l)
+
+    assert all(scalar == 3)
+    assert np.all(vector == np.arange(1, n + 1)*2)
+    assert np.all(matrix == np.sum(np.mgrid[1:4,2:8:2], axis=0))
+
+
 @pytest.mark.parametrize('frontend', available_frontends())
 @pytest.mark.parametrize('calls_only', (False, True))
 def test_transform_explicit_dimensions(tmp_path, frontend, builder, calls_only):
     """
-    Test flattening or arrays, meaning converting multi-dimensional
-    arrays to one-dimensional arrays including corresponding
-    index arithmetic (for calls).
+    Test making dimensions of arrays explicit and undoing this,
+    thus removing colon notation from array dimensions either for all
+    or for arrays within (inline) calls only.
     """
     fcode_driver = """
   SUBROUTINE driver_routine(nlon, nlev, a, b)
-    use kernel_mod, only: kernel_routine
+    use kernel_explicit_dimensions_mod, only: kernel_routine
     INTEGER, INTENT(IN)    :: nlon, nlev
     INTEGER, INTENT(INOUT) :: a(nlon,nlev)
     INTEGER, INTENT(INOUT)  :: b(nlon,nlev)
 
-    call kernel_routine(nlon, nlev, a, b)
+    call kernel_routine(nlon, a, b=b, nlev=nlev)
 
   END SUBROUTINE driver_routine
     """
+
     fcode_kernel = """
-  module kernel_mod
+  module kernel_explicit_dimensions_mod
   IMPLICIT NONE
-  CONTAINS
-  SUBROUTINE kernel_routine(nlon, nlev, a, b)
+  CONTAINS 
+  SUBROUTINE kernel_routine(nlon, a, b, nlev)
     INTEGER, INTENT(IN)    :: nlon, nlev
     INTEGER, INTENT(INOUT) :: a(nlon,nlev)
     INTEGER, INTENT(INOUT) :: b(nlon,nlev)
 
-    a = a + b
+    A = MYADD(A, B=B)
   END SUBROUTINE kernel_routine
-  end module kernel_mod
+
+  PURE ELEMENTAL FUNCTION MYADD(A, B)
+    INTEGER :: MYADD
+    INTEGER, INTENT(IN) :: A, B
+
+    MYADD = A + B
+  END FUNCTION
+  end module kernel_explicit_dimensions_mod
     """
+
     def init_arguments(nlon, nlev):
         a = 2*np.ones(shape=(nlon,nlev,), order='F', dtype=np.int32)
         b = 3*np.ones(shape=(nlon,nlev,), order='F', dtype=np.int32)
@@ -1101,7 +1227,7 @@ def test_transform_explicit_dimensions(tmp_path, frontend, builder, calls_only):
 
     kernel_module = Module.from_source(fcode_kernel, frontend=frontend, xmods=[tmp_path])
     driver = Subroutine.from_source(fcode_driver, frontend=frontend, xmods=[tmp_path],
-            definitions=[kernel_module])
+                                     definitions=[kernel_module])
     kernel = kernel_module.subroutines[0]
 
     # compile and test reference
@@ -1125,13 +1251,20 @@ def test_transform_explicit_dimensions(tmp_path, frontend, builder, calls_only):
     # remove explicit array dimensions (possibly only for calls)
     remove_explicit_array_dimensions(driver, calls_only=calls_only)
     remove_explicit_array_dimensions(kernel, calls_only=calls_only)
+
     kernel_call = FindNodes(CallStatement).visit(driver.body)[0]
     kernel_call_array_args = [arg for arg in kernel_call.arguments if isinstance(arg, sym.Array)]
     assert all(not arg.dimensions for arg in kernel_call_array_args)
-    kernel_arrays = FindVariables().visit(kernel.body)
     if calls_only:
-        assert all(len(arr.dimensions) == 2 for arr in kernel_arrays)
+        assignments = FindNodes(Assignment).visit(kernel.body)
+        assert len(assignments) == 1
+        assert len(assignments[0].lhs.dimensions) == 2
+        parameters = (assignments[0].rhs.parameters[0],)
+        parameters += (assignments[0].rhs.kwarguments[0][1],)
+        assert not parameters[0].dimensions
+        assert not parameters[1].dimensions
     else:
+        kernel_arrays = FindVariables().visit(kernel.body)
         assert all(not arr.dimensions for arr in kernel_arrays)
 
     # compile and test the resulting code
