@@ -13,14 +13,15 @@ import numpy as np
 from loki import Subroutine
 from loki.build import jit_compile, clean_test
 from loki.expression import symbols as sym
-from loki.frontend import available_frontends, OMNI
+from loki.frontend import available_frontends
 from loki.ir import (
     is_loki_pragma, pragmas_attached, FindNodes, Loop, Conditional,
-    Assignment
+    Assignment, FindVariables, nodes as ir
 )
 
 from loki.transformations.transform_loop import (
-    loop_interchange, loop_fusion, loop_fission, loop_unroll
+    do_loop_interchange, do_loop_fusion, do_loop_fission, do_loop_unroll,
+    TransformLoopsTransformation
 )
 
 
@@ -66,7 +67,7 @@ end subroutine transform_loop_interchange_plain
     assert np.all(a == ref)
 
     # Apply transformation
-    loop_interchange(routine)
+    do_loop_interchange(routine)
 
     interchanged_filepath = tmp_path/(f'{routine.name}_interchanged_{frontend}.f90')
     interchanged_function = jit_compile(routine, filepath=interchanged_filepath, objname=routine.name)
@@ -137,7 +138,7 @@ end subroutine transform_loop_interchange
     assert np.all(a == ref)
 
     # Apply transformation
-    loop_interchange(routine)
+    do_loop_interchange(routine)
 
     interchanged_filepath = tmp_path/(f'{routine.name}_interchanged_{frontend}.f90')
     interchanged_function = jit_compile(routine, filepath=interchanged_filepath, objname=routine.name)
@@ -197,7 +198,7 @@ end subroutine transform_loop_interchange_project
     assert np.all(a == ref)
 
     # Apply transformation
-    loop_interchange(routine, project_bounds=True)
+    do_loop_interchange(routine, project_bounds=True)
 
     interchanged_filepath = tmp_path/(f'{routine.name}_interchanged_{frontend}.f90')
     interchanged_function = jit_compile(routine, filepath=interchanged_filepath, objname=routine.name)
@@ -215,6 +216,59 @@ end subroutine transform_loop_interchange_project
     clean_test(interchanged_filepath)
 
 
+@pytest.mark.parametrize('frontend', available_frontends())
+@pytest.mark.parametrize('insert_loc', (False, True))
+def test_transform_loop_fuse_ordering(frontend, insert_loc):
+    """
+    Apply loop fusion for two loops with matching iteration spaces.
+    """
+    fcode = f"""
+subroutine transform_loop_fuse_ordering(a, b, c, n, m)
+  integer, intent(out) :: a(m, n), b(m, n), c(m)
+  integer, intent(in) :: n, m
+  integer :: i
+
+  !$loki loop-fusion group(1)
+  !$loki loop-interchange
+  do j=1,m
+    do i=1,n
+      a(j, i) = i + j
+    enddo
+  end do
+
+  !$loki loop-fusion group(1)
+  do i=1,n
+    do j=1,m
+      a(j, i) = i + j
+    enddo
+  end do
+
+  do j=1,m
+    c(j) = j
+  enddo
+
+  !$loki loop-fusion group(1) {'insert-loc' if insert_loc else ''}
+  do i=1,n-1
+    do j=1,m
+      b(j, i) = n-i+1 + j
+    enddo
+  end do
+end subroutine transform_loop_fuse_ordering
+"""
+    routine = Subroutine.from_source(fcode, frontend=frontend)
+    assert len(FindNodes(Loop).visit(routine.body)) == 7
+    do_loop_interchange(routine)
+    do_loop_fusion(routine)
+    loops = FindNodes(Loop).visit(routine.body)
+    assert len(loops) == 5
+    loop_0_vars = [var.name.lower() for var in FindVariables().visit(loops[0].body)]
+    if insert_loc:
+        assert loops[0].variable.name.lower() == 'j'
+        assert 'c' in loop_0_vars
+    else:
+        assert loops[0].variable.name.lower() == 'i'
+        assert 'c' not in loop_0_vars
+
 @pytest.mark.parametrize('frontend', available_frontends())
 def test_transform_loop_fuse_matching(tmp_path, frontend):
     """
@@ -251,7 +305,7 @@ end subroutine transform_loop_fuse_matching
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 2
-    loop_fusion(routine)
+    do_loop_fusion(routine)
     assert len(FindNodes(Loop).visit(routine.body)) == 1
 
     fused_filepath = tmp_path/(f'{routine.name}_fused_{frontend}.f90')
@@ -313,7 +367,7 @@ end subroutine transform_loop_fuse_subranges
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 3
-    loop_fusion(routine)
+    do_loop_fusion(routine)
     assert len(FindNodes(Loop).visit(routine.body)) == 1
 
     fused_filepath = tmp_path/(f'{routine.name}_fused_{frontend}.f90')
@@ -385,7 +439,7 @@ end subroutine transform_loop_fuse_groups
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 5
-    loop_fusion(routine)
+    do_loop_fusion(routine)
     assert len(FindNodes(Loop).visit(routine.body)) == 2
 
     fused_filepath = tmp_path/(f'{routine.name}_fused_{frontend}.f90')
@@ -428,7 +482,7 @@ end subroutine transform_loop_fuse_failures
 """
     routine = Subroutine.from_source(fcode, frontend=frontend)
     with pytest.raises(RuntimeError):
-        loop_fusion(routine)
+        do_loop_fusion(routine)
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
@@ -464,7 +518,7 @@ end subroutine transform_loop_fuse_alignment
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 2
-    loop_fusion(routine)
+    do_loop_fusion(routine)
     assert len(FindNodes(Loop).visit(routine.body)) == 1
 
     fused_filepath = tmp_path/(f'{routine.name}_fused_{frontend}.f90')
@@ -514,7 +568,7 @@ end subroutine transform_loop_fuse_nonmatching_lower
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 2
-    loop_fusion(routine)
+    do_loop_fusion(routine)
 
     loops = FindNodes(Loop).visit(routine.body)
     assert len(loops) == 1
@@ -570,7 +624,7 @@ end subroutine transform_loop_fuse_nonmatching_lower_annotated
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 2
-    loop_fusion(routine)
+    do_loop_fusion(routine)
 
     loops = FindNodes(Loop).visit(routine.body)
     assert len(loops) == 1
@@ -626,7 +680,7 @@ end subroutine transform_loop_fuse_nonmatching_upper
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 2
-    loop_fusion(routine)
+    do_loop_fusion(routine)
 
     loops = FindNodes(Loop).visit(routine.body)
     assert len(loops) == 1
@@ -687,7 +741,7 @@ end subroutine transform_loop_fuse_collapse
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 4
-    loop_fusion(routine)
+    do_loop_fusion(routine)
     loops = FindNodes(Loop).visit(routine.body)
     assert len(loops) == 2
     assert all(loop.bounds.start == '1' for loop in loops)
@@ -748,7 +802,7 @@ end subroutine transform_loop_fuse_collapse_nonmatching
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 4
-    loop_fusion(routine)
+    do_loop_fusion(routine)
     loops = FindNodes(Loop).visit(routine.body)
     assert len(loops) == 2
     assert all(loop.bounds.start == '1' for loop in loops)
@@ -810,7 +864,7 @@ end subroutine transform_loop_fuse_collapse_range
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 4
-    loop_fusion(routine)
+    do_loop_fusion(routine)
     loops = FindNodes(Loop).visit(routine.body)
     assert len(loops) == 2
     assert all(loop.bounds.start == '1' for loop in loops)
@@ -863,7 +917,7 @@ end subroutine transform_loop_fission_single
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 1
-    loop_fission(routine)
+    do_loop_fission(routine)
 
     loops = FindNodes(Loop).visit(routine.body)
     assert len(loops) == 2
@@ -918,7 +972,7 @@ end subroutine transform_loop_fission_nested
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 1
     assert len(FindNodes(Conditional).visit(routine.body)) == 1
-    loop_fission(routine)
+    do_loop_fission(routine)
 
     loops = FindNodes(Loop).visit(routine.body)
     assert len(loops) == 2
@@ -978,7 +1032,7 @@ end subroutine transform_loop_fission_nested_promote
     assert len(FindNodes(Loop).visit(routine.body)) == 1
     assert len(FindNodes(Conditional).visit(routine.body)) == 2
     assert len(FindNodes(Assignment).visit(routine.body)) == 3
-    loop_fission(routine)
+    do_loop_fission(routine)
 
     loops = FindNodes(Loop).visit(routine.body)
     assert len(loops) == 2
@@ -1043,7 +1097,7 @@ end subroutine transform_loop_fission_collapse
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 2
     assert len(FindNodes(Assignment).visit(routine.body)) == 8
-    loop_fission(routine)
+    do_loop_fission(routine)
 
     loops = FindNodes(Loop).visit(routine.body)
     assert len(loops) == 8
@@ -1098,7 +1152,7 @@ end subroutine transform_loop_fission_multiple
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 1
-    loop_fission(routine)
+    do_loop_fission(routine)
 
     loops = FindNodes(Loop).visit(routine.body)
     assert len(loops) == 3
@@ -1153,7 +1207,7 @@ end subroutine transform_loop_fission_promote
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 1
-    loop_fission(routine)
+    do_loop_fission(routine)
 
     loops = FindNodes(Loop).visit(routine.body)
     assert len(loops) == 2
@@ -1212,7 +1266,7 @@ end subroutine transform_loop_fission_promote_conflicting_lengths
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 2
-    loop_fission(routine)
+    do_loop_fission(routine)
 
     loops = FindNodes(Loop).visit(routine.body)
     assert len(loops) == 4
@@ -1269,7 +1323,7 @@ end subroutine transform_loop_fission_promote_array
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 2
-    loop_fission(routine)
+    do_loop_fission(routine)
 
     loops = FindNodes(Loop).visit(routine.body)
     assert len(loops) == 3
@@ -1322,7 +1376,7 @@ end subroutine transform_loop_fission_promote_multiple
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 2
-    loop_fission(routine)
+    do_loop_fission(routine)
 
     loops = FindNodes(Loop).visit(routine.body)
     assert len(loops) == 3
@@ -1388,7 +1442,7 @@ end subroutine transform_loop_fission_multiple_promote
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 5
-    loop_fission(routine)
+    do_loop_fission(routine)
 
     loops = FindNodes(Loop).visit(routine.body)
     assert len(loops) == 8
@@ -1447,7 +1501,7 @@ end subroutine transform_loop_fission_promote_read_after_write
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 2
-    loop_fission(routine)
+    do_loop_fission(routine)
 
     loops = FindNodes(Loop).visit(routine.body)
     assert len(loops) == 3
@@ -1514,7 +1568,7 @@ end subroutine transform_loop_fission_promote_mult_r_a_w
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 5
-    loop_fission(routine)
+    do_loop_fission(routine)
 
     loops = FindNodes(Loop).visit(routine.body)
     assert len(loops) == 8
@@ -1581,9 +1635,9 @@ end subroutine transform_loop_fusion_fission
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 4
-    loop_fusion(routine)
+    do_loop_fusion(routine)
     assert len(FindNodes(Loop).visit(routine.body)) == 3
-    loop_fission(routine)
+    do_loop_fission(routine)
 
     loops = FindNodes(Loop).visit(routine.body)
     assert len(loops) == 4
@@ -1631,11 +1685,11 @@ end subroutine test_transform_loop_unroll
     # Test the reference solution
     s = np.zeros(1)
     function(s=s)
-    assert s == sum([x + 1 for x in range(1, 11)])
+    assert s == sum(x + 1 for x in range(1, 11))
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 1
-    loop_unroll(routine)
+    do_loop_unroll(routine)
     assert len(FindNodes(Loop).visit(routine.body)) == 0 and len(FindNodes(Assignment).visit(routine.body)) == 10
 
     unrolled_filepath = tmp_path / f'{routine.name}_unrolled_{frontend}.f90'
@@ -1644,7 +1698,7 @@ end subroutine test_transform_loop_unroll
     # Test transformation
     s = np.zeros(1)
     unrolled_function(s=s)
-    assert s == sum([x + 1 for x in range(1, 11)])
+    assert s == sum(x + 1 for x in range(1, 11))
 
     clean_test(filepath)
     clean_test(unrolled_filepath)
@@ -1673,11 +1727,11 @@ end subroutine test_transform_loop_unroll_step
     # Test the reference solution
     s = np.zeros(1)
     function(s=s)
-    assert s == sum([x + 1 for x in range(1, 11, 2)])
+    assert s == sum(x + 1 for x in range(1, 11, 2))
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 1
-    loop_unroll(routine)
+    do_loop_unroll(routine)
     assert len(FindNodes(Loop).visit(routine.body)) == 0 and len(FindNodes(Assignment).visit(routine.body)) == 5
 
     unrolled_filepath = tmp_path / f'{routine.name}_unrolled_{frontend}.f90'
@@ -1686,7 +1740,7 @@ end subroutine test_transform_loop_unroll_step
     # Test transformation
     s = np.zeros(1)
     unrolled_function(s=s)
-    assert s == sum([x + 1 for x in range(1, 11, 2)])
+    assert s == sum(x + 1 for x in range(1, 11, 2))
 
     clean_test(filepath)
     clean_test(unrolled_filepath)
@@ -1717,11 +1771,11 @@ end subroutine test_transform_loop_unroll_non_literal_range
     # Test the reference solution
     s = np.zeros(1)
     function(s=s)
-    assert s == sum([x + 1 for x in range(1, 11)])
+    assert s == sum(x + 1 for x in range(1, 11))
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 1
-    loop_unroll(routine)
+    do_loop_unroll(routine)
     assert len(FindNodes(Loop).visit(routine.body)) == 1 and len(FindNodes(Assignment).visit(routine.body)) == 2
 
     unrolled_filepath = tmp_path / f'{routine.name}_unrolled_{frontend}.f90'
@@ -1730,7 +1784,7 @@ end subroutine test_transform_loop_unroll_non_literal_range
     # Test transformation
     s = np.zeros(1)
     unrolled_function(s=s)
-    assert s == sum([x + 1 for x in range(1, 11)])
+    assert s == sum(x + 1 for x in range(1, 11))
 
     clean_test(filepath)
     clean_test(unrolled_filepath)
@@ -1762,11 +1816,11 @@ end subroutine test_transform_loop_unroll_nested
     # Test the reference solution
     s = np.zeros(1)
     function(s=s)
-    assert s == sum([a + b + 1 for (a, b) in itertools.product(range(1, 11), range(1, 6))])
+    assert s == sum(a + b + 1 for (a, b) in itertools.product(range(1, 11), range(1, 6)))
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 2
-    loop_unroll(routine)
+    do_loop_unroll(routine)
     assert len(FindNodes(Loop).visit(routine.body)) == 0 and len(FindNodes(Assignment).visit(routine.body)) == 50
 
     unrolled_filepath = tmp_path / f'{routine.name}_unrolled_{frontend}.f90'
@@ -1775,7 +1829,7 @@ end subroutine test_transform_loop_unroll_nested
     # Test transformation
     s = np.zeros(1)
     unrolled_function(s=s)
-    assert s == sum([a + b + 1 for (a, b) in itertools.product(range(1, 11), range(1, 6))])
+    assert s == sum(a + b + 1 for (a, b) in itertools.product(range(1, 11), range(1, 6)))
 
     clean_test(filepath)
     clean_test(unrolled_filepath)
@@ -1807,11 +1861,11 @@ end subroutine test_transform_loop_unroll_nested_restricted_depth
     # Test the reference solution
     s = np.zeros(1)
     function(s=s)
-    assert s == sum([a + b + 1 for (a, b) in itertools.product(range(1, 11), range(1, 6))])
+    assert s == sum(a + b + 1 for (a, b) in itertools.product(range(1, 11), range(1, 6)))
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 2
-    loop_unroll(routine)
+    do_loop_unroll(routine)
     assert len(FindNodes(Loop).visit(routine.body)) == 10 and len(FindNodes(Assignment).visit(routine.body)) == 10
 
     unrolled_filepath = tmp_path / f'{routine.name}_unrolled_{frontend}.f90'
@@ -1820,7 +1874,7 @@ end subroutine test_transform_loop_unroll_nested_restricted_depth
     # Test transformation
     s = np.zeros(1)
     unrolled_function(s=s)
-    assert s == sum([a + b + 1 for (a, b) in itertools.product(range(1, 11), range(1, 6))])
+    assert s == sum(a + b + 1 for (a, b) in itertools.product(range(1, 11), range(1, 6)))
 
     clean_test(filepath)
     clean_test(unrolled_filepath)
@@ -1854,11 +1908,11 @@ end subroutine test_transform_loop_unroll_nested_restricted_depth
     # Test the reference solution
     s = np.zeros(1)
     function(s=s)
-    assert s == sum([a + b + 1 for (a, b) in itertools.product(range(1, 11), range(1, 6))])
+    assert s == sum(a + b + 1 for (a, b) in itertools.product(range(1, 11), range(1, 6)))
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 2
-    loop_unroll(routine)
+    do_loop_unroll(routine)
     assert len(FindNodes(Loop).visit(routine.body)) == 1 and len(FindNodes(Assignment).visit(routine.body)) == 6
 
     unrolled_filepath = tmp_path / f'{routine.name}_unrolled_{frontend}.f90'
@@ -1867,7 +1921,7 @@ end subroutine test_transform_loop_unroll_nested_restricted_depth
     # Test transformation
     s = np.zeros(1)
     unrolled_function(s=s)
-    assert s == sum([a + b + 1 for (a, b) in itertools.product(range(1, 11), range(1, 6))])
+    assert s == sum(a + b + 1 for (a, b) in itertools.product(range(1, 11), range(1, 6)))
 
     clean_test(filepath)
     clean_test(unrolled_filepath)
@@ -1905,7 +1959,7 @@ end subroutine test_transform_loop_unroll_nested_counters
 
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 2
-    loop_unroll(routine)
+    do_loop_unroll(routine)
     assert len(FindNodes(Loop).visit(routine.body)) == 0 and \
            len(FindNodes(Assignment).visit(routine.body)) == len(tuples)
 
@@ -1915,7 +1969,7 @@ end subroutine test_transform_loop_unroll_nested_counters
     # Test transformation
     s = np.zeros(1)
     unrolled_function(s=s)
-    assert s == sum([a + b + 1 for (a, b) in itertools.product(range(1, 11), range(1, 11)) if b <= a])
+    assert s == sum(a + b + 1 for (a, b) in itertools.product(range(1, 11), range(1, 11)) if b <= a)
 
     clean_test(filepath)
     clean_test(unrolled_filepath)
@@ -1953,10 +2007,10 @@ end subroutine test_transform_loop_unroll_nested_neighbours
     # Test the reference solution
     s = np.zeros(1)
     function(s=s)
-    assert s == 2 * sum([a + b + 1 for (a, b) in itertools.product(range(1, 11), range(1, 6))])
+    assert s == 2 * sum(a + b + 1 for (a, b) in itertools.product(range(1, 11), range(1, 6)))
     # Apply transformation
     assert len(FindNodes(Loop).visit(routine.body)) == 3
-    loop_unroll(routine)
+    do_loop_unroll(routine)
     assert len(FindNodes(Loop).visit(routine.body)) == 10 and len(FindNodes(Assignment).visit(routine.body)) == 60
 
     unrolled_filepath = tmp_path / f'{routine.name}_unrolled_{frontend}.f90'
@@ -1965,7 +2019,89 @@ end subroutine test_transform_loop_unroll_nested_neighbours
     # Test transformation
     s = np.zeros(1)
     unrolled_function(s=s)
-    assert s == 2 * sum([a + b + 1 for (a, b) in itertools.product(range(1, 11), range(1, 6))])
+    assert s == 2 * sum(a + b + 1 for (a, b) in itertools.product(range(1, 11), range(1, 6)))
 
     clean_test(filepath)
     clean_test(unrolled_filepath)
+
+
+@pytest.mark.parametrize('frontend', available_frontends())
+@pytest.mark.parametrize('loop_interchange', [False, True])
+@pytest.mark.parametrize('loop_fusion', [False, True])
+@pytest.mark.parametrize('loop_fission', [False, True])
+@pytest.mark.parametrize('loop_unroll', [False, True])
+def test_transform_loop_transformation(frontend, loop_interchange, loop_fusion, loop_fission, loop_unroll):
+    fcode = """
+subroutine transform_loop()
+  integer, parameter :: m = 8
+  integer, parameter :: n = 16
+
+  integer :: array(m,n)
+  integer :: a(n), b(n)
+  integer :: i, j, s
+
+  !$loki loop-interchange
+  do i=1,n
+    do j=1,m
+      array(j, i) = i + j
+    end do
+  end do
+
+  !$loki loop-fusion
+  do i=1,n
+    a(i) = i
+  end do
+
+  !$loki loop-fusion
+  do i=1,n
+    b(i) = n-i+1
+  end do
+
+  do j=1,n
+    a(j) = j
+    !$loki loop-fission
+    b(j) = n-j
+  end do
+
+  !$loki loop-unroll
+  do i=1, 10
+      s = s + i + 1
+  end do
+end subroutine transform_loop
+    """
+
+    routine = Subroutine.from_source(fcode, frontend=frontend)
+    transform = TransformLoopsTransformation(loop_interchange=loop_interchange, loop_fusion=loop_fusion,
+                                             loop_fission=loop_fission, loop_unroll=loop_unroll)
+
+    num_pragmas = len(FindNodes(ir.Pragma).visit(routine.body))
+    num_loops = len(FindNodes(ir.Loop).visit(routine.body))
+
+    transform.apply(routine)
+    pragmas = FindNodes(ir.Pragma).visit(routine.body)
+    loops = FindNodes(ir.Loop).visit(routine.body)
+
+    if loop_interchange:
+        num_pragmas -= 1
+        assert loops[0].variable == 'j'
+        assert not any('loop-interchange' in pragma.content for pragma in pragmas)
+        assert FindNodes(ir.Loop).visit(loops[0].body)[0].variable == 'i'
+
+    if loop_fusion:
+        num_pragmas -= 1
+        num_loops -= 1
+        assert not any('loop-fusion' in pragma.content for pragma in pragmas)
+        assert len(FindNodes(ir.Assignment).visit(loops[2].body)) == 2
+
+    if loop_fission:
+        num_pragmas -= 1
+        num_loops += 1
+        assert not any('loop-fission' in pragma.content for pragma in pragmas)
+
+    if loop_unroll:
+        num_pragmas -= 1
+        num_loops -= 1
+        assert not any('loop-unroll' in pragma.content for pragma in pragmas)
+
+    assert len(loops) == num_loops
+    assert len(pragmas) == num_pragmas
