@@ -19,14 +19,14 @@ from loki.analyse import (
     loop_carried_dependencies
 )
 from loki.expression import (
-    symbols as sym, SubstituteExpressions, FindVariables,
-    simplify, is_constant, symbolic_op, parse_expr, IntLiteral,
-    FloatLiteral
+    symbols as sym, simplify, is_constant, symbolic_op, parse_expr,
+    IntLiteral, FloatLiteral
 )
 from loki.ir import (
     Loop, Conditional, Comment, Pragma, FindNodes, Transformer,
     NestedMaskedTransformer, is_parent_of, is_loki_pragma,
-    get_pragma_parameters, pragmas_attached
+    get_pragma_parameters, pragmas_attached, SubstituteExpressions,
+    FindVariables
 )
 from loki.logging import info, warning
 from loki.tools import (
@@ -36,9 +36,11 @@ from loki.tools import (
 from loki.transformations.array_indexing import (
     promotion_dimensions_from_loop_nest, promote_nonmatching_variables
 )
+from loki.batch import Transformation
 
 
-__all__ = ['loop_interchange', 'loop_fusion', 'loop_fission', 'loop_unroll']
+__all__ = ['do_loop_interchange', 'do_loop_fusion', 'do_loop_fission', 'do_loop_unroll',
+           'TransformLoopsTransformation']
 
 
 from loki.analyse.util_polyhedron import Polyhedron
@@ -181,7 +183,7 @@ def get_loop_components(loops):
     return (as_tuple(loop_variables), as_tuple(loop_ranges), as_tuple(loop_bodies))
 
 
-def loop_interchange(routine, project_bounds=False):
+def do_loop_interchange(routine, project_bounds=False):
     """
     Search for loops annotated with the `loki loop-interchange` pragma and attempt
     to reorder them.
@@ -279,7 +281,7 @@ def pragma_ranges_to_loop_ranges(parameters, scope):
     return as_tuple(ranges)
 
 
-def loop_fusion(routine):
+def do_loop_fusion(routine):
     """
     Search for loops annotated with the `loki loop-fusion` pragma and attempt
     to fuse them into a single loop.
@@ -294,140 +296,147 @@ def loop_fusion(routine):
                 group = parameters.get('group', 'default')
                 fusion_groups[group] += [(loop, parameters)]
 
-    if not fusion_groups:
-        return
+        if not fusion_groups:
+            return
 
-    # Merge loops in each group and put them in the position of the group's first loop
-    for group, loop_parameter_lists in fusion_groups.items():
-        loop_list, parameters = zip(*loop_parameter_lists)
+        # Merge loops in each group and put them in the position of the group's first loop
+        #  UNLESS 'insert-loc' location is specified for at least one of the group's fusion
+        #  pragmas, in this case the position is the first occurence of 'insert-loc' for each group
+        for group, loop_parameter_lists in fusion_groups.items():
+            loop_list, parameters = zip(*loop_parameter_lists)
 
-        # First, determine the collapse depth and extract user-annotated loop ranges from pragmas
-        collapse = [param.get('collapse', None) for param in parameters]
-        if collapse != [collapse[0]] * len(collapse):
-            raise RuntimeError(f'Conflicting collapse values in group "{group}"')
-        collapse = int(collapse[0]) if collapse[0] is not None else 1
+            # First, determine the collapse depth and extract user-annotated loop ranges from pragmas
+            collapse = [param.get('collapse', None) for param in parameters]
+            insert_locs = [param.get('insert-loc', False) for param in parameters]
+            if collapse != [collapse[0]] * len(collapse):
+                raise RuntimeError(f'Conflicting collapse values in group "{group}"')
+            collapse = int(collapse[0]) if collapse[0] is not None else 1
 
-        pragma_ranges = [pragma_ranges_to_loop_ranges(param, routine) for param in parameters]
+            pragma_ranges = [pragma_ranges_to_loop_ranges(param, routine) for param in parameters]
 
-        # If we have a pragma somewhere with an explicit loop range, we use that for the fused loop
-        range_set = {r for r in pragma_ranges if r is not None}
-        if len(range_set) not in (0, 1):
-            raise RuntimeError(f'Pragma-specified loop ranges in group "{group}" do not match')
+            # If we have a pragma somewhere with an explicit loop range, we use that for the fused loop
+            range_set = {r for r in pragma_ranges if r is not None}
+            if len(range_set) not in (0, 1):
+                raise RuntimeError(f'Pragma-specified loop ranges in group "{group}" do not match')
 
-        fusion_ranges = None
-        if range_set:
-            fusion_ranges = range_set.pop()
+            fusion_ranges = None
+            if range_set:
+                fusion_ranges = range_set.pop()
 
-        # Next, extract loop ranges for all loops in group and convert to iteration space
-        # polyhedrons for easier alignment
-        loop_variables, loop_ranges, loop_bodies = \
-                zip(*[get_loop_components(get_nested_loops(loop, collapse)) for loop in loop_list])
-        iteration_spaces = [Polyhedron.from_loop_ranges(variables, ranges)
-                            for variables, ranges in zip(loop_variables, loop_ranges)]
+            # Next, extract loop ranges for all loops in group and convert to iteration space
+            # polyhedrons for easier alignment
+            loop_variables, loop_ranges, loop_bodies = \
+                    zip(*[get_loop_components(get_nested_loops(loop, collapse)) for loop in loop_list])
+            iteration_spaces = [Polyhedron.from_loop_ranges(variables, ranges)
+                                for variables, ranges in zip(loop_variables, loop_ranges)]
 
-        # Find the fused iteration space (if not given by a pragma)
-        if fusion_ranges is None:
-            fusion_ranges = []
-            for level in range(collapse):
-                lower_bounds, upper_bounds = [], []
-                ignored_variables = list(range(level+1, collapse))
+            # Find the fused iteration space (if not given by a pragma)
+            if fusion_ranges is None:
+                fusion_ranges = []
+                for level in range(collapse):
+                    lower_bounds, upper_bounds = [], []
+                    ignored_variables = list(range(level+1, collapse))
 
-                for p in iteration_spaces:
-                    for bound in p.lower_bounds(level, ignored_variables):
-                        # Decide if we learn something new from this bound, which could be because:
-                        # (1) we don't have any bounds, yet
-                        # (2) bound is smaller than existing lower bounds (i.e. diff < 0)
-                        # (3) bound is not constant and none of the existing bounds are lower (i.e. diff >= 0)
-                        diff = [simplify(bound - b) for b in lower_bounds]
-                        is_any_negative = any(is_constant(d) and symbolic_op(d, op.lt, 0) for d in diff)
-                        is_any_not_negative = any(is_constant(d) and symbolic_op(d, op.ge, 0) for d in diff)
-                        is_new_bound = (not lower_bounds or is_any_negative or
-                                        (not is_constant(bound) and not is_any_not_negative))
-                        if is_new_bound:
-                            # Remove any lower bounds made redundant by bound:
-                            lower_bounds = [b for b, d in zip(lower_bounds, diff)
-                                            if not (is_constant(d) and symbolic_op(d, op.lt, 0))]
-                            lower_bounds += [bound]
+                    for p in iteration_spaces:
+                        for bound in p.lower_bounds(level, ignored_variables):
+                            # Decide if we learn something new from this bound, which could be because:
+                            # (1) we don't have any bounds, yet
+                            # (2) bound is smaller than existing lower bounds (i.e. diff < 0)
+                            # (3) bound is not constant and none of the existing bounds are lower (i.e. diff >= 0)
+                            diff = [simplify(bound - b) for b in lower_bounds]
+                            is_any_negative = any(is_constant(d) and symbolic_op(d, op.lt, 0) for d in diff)
+                            is_any_not_negative = any(is_constant(d) and symbolic_op(d, op.ge, 0) for d in diff)
+                            is_new_bound = (not lower_bounds or is_any_negative or
+                                            (not is_constant(bound) and not is_any_not_negative))
+                            if is_new_bound:
+                                # Remove any lower bounds made redundant by bound:
+                                lower_bounds = [b for b, d in zip(lower_bounds, diff)
+                                                if not (is_constant(d) and symbolic_op(d, op.lt, 0))]
+                                lower_bounds += [bound]
 
-                    for bound in p.upper_bounds(level, ignored_variables):
-                        # Decide if we learn something new from this bound, which could be because:
-                        # (1) we don't have any bounds, yet
-                        # (2) bound is larger than existing upper bounds (i.e. diff > 0)
-                        # (3) bound is not constant and none of the existing bounds are larger (i.e. diff <= 0)
-                        diff = [simplify(bound - b) for b in upper_bounds]
-                        is_any_positive = any(is_constant(d) and symbolic_op(d, op.gt, 0) for d in diff)
-                        is_any_not_positive = any(is_constant(d) and symbolic_op(d, op.le, 0) for d in diff)
-                        is_new_bound = (not upper_bounds or is_any_positive or
-                                        (not is_constant(bound) and not is_any_not_positive))
-                        if is_new_bound:
-                            # Remove any lower bounds made redundant by bound:
-                            upper_bounds = [b for b, d in zip(upper_bounds, diff)
-                                            if not (is_constant(d) and symbolic_op(d, op.gt, 0))]
-                            upper_bounds += [bound]
+                        for bound in p.upper_bounds(level, ignored_variables):
+                            # Decide if we learn something new from this bound, which could be because:
+                            # (1) we don't have any bounds, yet
+                            # (2) bound is larger than existing upper bounds (i.e. diff > 0)
+                            # (3) bound is not constant and none of the existing bounds are larger (i.e. diff <= 0)
+                            diff = [simplify(bound - b) for b in upper_bounds]
+                            is_any_positive = any(is_constant(d) and symbolic_op(d, op.gt, 0) for d in diff)
+                            is_any_not_positive = any(is_constant(d) and symbolic_op(d, op.le, 0) for d in diff)
+                            is_new_bound = (not upper_bounds or is_any_positive or
+                                            (not is_constant(bound) and not is_any_not_positive))
+                            if is_new_bound:
+                                # Remove any lower bounds made redundant by bound:
+                                upper_bounds = [b for b, d in zip(upper_bounds, diff)
+                                                if not (is_constant(d) and symbolic_op(d, op.gt, 0))]
+                                upper_bounds += [bound]
 
-                if len(lower_bounds) == 1:
-                    lower_bounds = lower_bounds[0]
-                else:
-                    fct_symbol = sym.ProcedureSymbol('min', scope=routine)
-                    lower_bounds = sym.InlineCall(fct_symbol, parameters=as_tuple(lower_bounds))
+                    if len(lower_bounds) == 1:
+                        lower_bounds = lower_bounds[0]
+                    else:
+                        # TODO: could/should be ProcedureSymbol, however refer to issue: #390
+                        fct_symbol = sym.DeferredTypeSymbol(name='min', scope=routine)
+                        lower_bounds = sym.InlineCall(fct_symbol, parameters=as_tuple(lower_bounds))
 
-                if len(upper_bounds) == 1:
-                    upper_bounds = upper_bounds[0]
-                else:
-                    fct_symbol = sym.ProcedureSymbol('max', scope=routine)
-                    upper_bounds = sym.InlineCall(fct_symbol, parameters=as_tuple(upper_bounds))
+                    if len(upper_bounds) == 1:
+                        upper_bounds = upper_bounds[0]
+                    else:
+                        # TODO: could/should be ProcedureSymbol, however refer to issue: #390
+                        fct_symbol = sym.DeferredTypeSymbol(name='max', scope=routine)
+                        upper_bounds = sym.InlineCall(fct_symbol, parameters=as_tuple(upper_bounds))
 
-                fusion_ranges += [sym.LoopRange((lower_bounds, upper_bounds))]
+                    fusion_ranges += [sym.LoopRange((lower_bounds, upper_bounds))]
 
-        # Align loop ranges and collect bodies
-        fusion_bodies = []
-        fusion_variables = loop_variables[0]
-        for idx, (variables, ranges, bodies, p) in enumerate(
-                zip(loop_variables, loop_ranges, loop_bodies, iteration_spaces)):
-            # TODO: This throws away anything that is not in the inner-most loop body.
-            body = flatten([Comment(f'! Loki loop-fusion - body {idx} begin'),
-                            bodies[-1],
-                            Comment(f'! Loki loop-fusion - body {idx} end')])
+            # Align loop ranges and collect bodies
+            fusion_bodies = []
+            fusion_variables = loop_variables[0]
+            for idx, (variables, ranges, bodies, p) in enumerate(
+                    zip(loop_variables, loop_ranges, loop_bodies, iteration_spaces)):
+                # TODO: This throws away anything that is not in the inner-most loop body.
+                body = flatten([Comment(f'! Loki loop-fusion - body {idx} begin'),
+                                bodies[-1],
+                                Comment(f'! Loki loop-fusion - body {idx} end')])
 
-            # Replace loop variables if necessary
-            var_map = {}
-            for loop_variable, fusion_variable in zip(variables, fusion_variables):
-                if loop_variable != fusion_variable:
-                    var_map.update({var: fusion_variable for var in FindVariables().visit(body)
-                                    if var.name.lower() == loop_variable.name})
-            if var_map:
-                body = SubstituteExpressions(var_map).visit(body)
+                # Replace loop variables if necessary
+                var_map = {}
+                for loop_variable, fusion_variable in zip(variables, fusion_variables):
+                    if loop_variable != fusion_variable:
+                        var_map.update({var: fusion_variable for var in FindVariables().visit(body)
+                                        if var.name.lower() == loop_variable.name})
+                if var_map:
+                    body = SubstituteExpressions(var_map).visit(body)
 
-            # Wrap in conditional if loop bounds are different
-            conditions = []
-            for loop_range, fusion_range, variable in zip(ranges, fusion_ranges, fusion_variables):
-                if symbolic_op(loop_range.start, op.ne, fusion_range.start):
-                    conditions += [sym.Comparison(variable, '>=', loop_range.start)]
-                if symbolic_op(loop_range.stop, op.ne, fusion_range.stop):
-                    conditions += [sym.Comparison(variable, '<=', loop_range.stop)]
-            if conditions:
-                if len(conditions) == 1:
-                    condition = conditions[0]
-                else:
-                    condition = sym.LogicalAnd(as_tuple(conditions))
-                body = Conditional(condition=condition, body=as_tuple(body), else_body=())
+                # Wrap in conditional if loop bounds are different
+                conditions = []
+                for loop_range, fusion_range, variable in zip(ranges, fusion_ranges, fusion_variables):
+                    if symbolic_op(loop_range.start, op.ne, fusion_range.start):
+                        conditions += [sym.Comparison(variable, '>=', loop_range.start)]
+                    if symbolic_op(loop_range.stop, op.ne, fusion_range.stop):
+                        conditions += [sym.Comparison(variable, '<=', loop_range.stop)]
+                if conditions:
+                    if len(conditions) == 1:
+                        condition = conditions[0]
+                    else:
+                        condition = sym.LogicalAnd(as_tuple(conditions))
+                    body = Conditional(condition=condition, body=as_tuple(body), else_body=())
 
-            fusion_bodies += [body]
+                fusion_bodies += [body]
 
-        # Create the nested fused loop and replace original loops
-        fusion_loop = flatten(fusion_bodies)
-        for fusion_variable, fusion_range in zip(reversed(fusion_variables), reversed(fusion_ranges)):
-            fusion_loop = Loop(variable=fusion_variable, body=as_tuple(fusion_loop), bounds=fusion_range)
+            # Create the nested fused loop and replace original loops
+            fusion_loop = flatten(fusion_bodies)
+            for fusion_variable, fusion_range in zip(reversed(fusion_variables), reversed(fusion_ranges)):
+                fusion_loop = Loop(variable=fusion_variable, body=as_tuple(fusion_loop), bounds=fusion_range)
 
-        comment = Comment(f'! Loki loop-fusion group({group})')
-        loop_map[loop_list[0]] = (comment, fusion_loop)
-        comment = Comment(f'! Loki loop-fusion group({group}) - loop hoisted')
-        loop_map.update({loop: comment for loop in loop_list[1:]})
+            comment = Comment(f'! Loki loop-fusion group({group})')
+            insert_loc = insert_locs.index(None) if None in insert_locs else 0
+            loop_map[loop_list[insert_loc]] = (comment, Pragma(keyword='loki',
+                content=f'fused-loop group({group})'), fusion_loop)
+            comment = Comment(f'! Loki loop-fusion group({group}) - loop hoisted')
+            loop_map.update({loop: comment for i_loop, loop in enumerate(loop_list) if i_loop != insert_loc})
 
-    # Apply transformation
-    routine.body = Transformer(loop_map).visit(routine.body)
-    info('%s: fused %d loops in %d groups.', routine.name,
-         sum(len(loop_list) for loop_list in fusion_groups.values()), len(fusion_groups))
+        # Apply transformation
+        routine.body = Transformer(loop_map).visit(routine.body)
+        info('%s: fused %d loops in %d groups.', routine.name,
+             sum(len(loop_list) for loop_list in fusion_groups.values()), len(fusion_groups))
 
 
 class FissionTransformer(NestedMaskedTransformer):
@@ -516,7 +525,7 @@ class FissionTransformer(NestedMaskedTransformer):
         return as_tuple(i for i in rebuilt if i)
 
 
-def loop_fission(routine, promote=True, warn_loop_carries=True):
+def do_loop_fission(routine, promote=True, warn_loop_carries=True):
     """
     Search for ``!$loki loop-fission`` pragmas in loops and split them.
 
@@ -679,7 +688,7 @@ class LoopUnrollTransformer(Transformer):
                     )
 
 
-def loop_unroll(routine, warn_iterations_length=True):
+def do_loop_unroll(routine, warn_iterations_length=True):
     """
     Search for ``!$loki loop-unroll`` pragmas in loops and unroll them.
 
@@ -745,3 +754,76 @@ def loop_unroll(routine, warn_iterations_length=True):
 
     with pragmas_attached(routine, Loop):
         routine.body = PragmaLoopUnrollTransformer(warn_iterations_length=warn_iterations_length).visit(routine.body)
+
+
+class TransformLoopsTransformation(Transformation):
+    """
+    A :any:`Transformation` that provides a common location for the various loop transformations to be called
+    in a :any:`Scheduler` pipeline.
+
+    The transformation applies the following methods in order:
+
+    * :any:`do_loop_interchange`
+    * :any:`do_loop_fusion`
+    * :any:`do_loop_fission`
+    * :any:`do_loop_unroll`
+
+    Parameters
+    ----------
+    loop_interchange : bool
+        Run the ``do_loop_interchange`` utility. Default: ``False``.
+    loop_fusion : bool
+        Run the ``do_loop_fusion`` utility. Default: ``False``.
+    loop_fission : bool
+        Run the ``do_loop_fission`` utility. Default: ``False``.
+    loop_unroll : bool
+        Run the ``do_loop_unroll`` utility. Default: ``False``.
+    interchange_project_bounds : bool
+        Project loop bounds whilst performing loop interchange. Default: ``False``.
+    fission_promote : bool
+        Try to automatically detect read-after-write across fission points
+        and promote corresponding variables. Note that this does not affect
+        promotion of variables listed directly in the pragma's ``promote``
+        option. Default: ``True``.
+    fission_warn_loop_carries : bool
+        Try to automatically detect loop-carried dependencies and warn
+        when the fission point sits after the initial read and before the
+        final write. Default: ``True``.
+    unroll_warn_iterations_length : bool
+        This specifies if warnings should be generated when unrolling
+        loops with a large number of iterations (32). It's mainly to
+        disable warnings when loops are being unrolled for internal
+        transformations and analysis. Default: ``True``.
+    """
+
+    def __init__(
+            self, loop_interchange=False, loop_fusion=False, loop_fission=False,
+            loop_unroll=False, interchange_project_bounds=False, fission_promote=True,
+            fission_warn_loop_carries=True, unroll_warn_iterations_length=True
+    ):
+        self.loop_interchange = loop_interchange
+        self.loop_fusion = loop_fusion
+        self.loop_fission = loop_fission
+        self.loop_unroll = loop_unroll
+        self.interchange_project_bounds = interchange_project_bounds
+        self.fission_promote = fission_promote
+        self.fission_warn_loop_carries = fission_warn_loop_carries
+        self.unroll_warn_iterations_length = unroll_warn_iterations_length
+
+    def transform_subroutine(self, routine, **kwargs):
+
+        # Interchange loops
+        if self.loop_interchange:
+            do_loop_interchange(routine, project_bounds=self.interchange_project_bounds)
+
+        # Fuse loops
+        if self.loop_fusion:
+            do_loop_fusion(routine)
+
+        # Split loops
+        if self.loop_fission:
+            do_loop_fission(routine, promote=self.fission_promote, warn_loop_carries=self.fission_warn_loop_carries)
+
+        # Unroll loops
+        if self.loop_unroll:
+            do_loop_unroll(routine, warn_iterations_length=self.unroll_warn_iterations_length)
