@@ -6,25 +6,35 @@
 # nor does it submit to any jurisdiction.
 
 from pathlib import Path
-# from shutil import rmtree
 import pytest
 import numpy as np
 
 from loki import Subroutine, Module, cgen, cppgen, cudagen, FindNodes, Dimension, Scheduler, read_file
 from loki.build import jit_compile, jit_compile_lib, clean_test, Builder, Obj
 import loki.expression.symbols as sym
-from loki.frontend import available_frontends, OFP
+from loki.frontend import available_frontends
 from loki import ir
 
 from loki.transformations.transpile import FortranCTransformation
 from loki.transformations.single_column import SCCLowLevelHoist, SCCLowLevelParametrise
 
+# pylint: disable=too-many-lines
+
 @pytest.fixture(scope='function', name='builder')
 def fixture_builder(tmp_path):
     yield Builder(source_dirs=tmp_path, build_dir=tmp_path)
     Obj.clear_cache()
 
 
+def test_transpile_unsupported_lang():
+    """
+    A simple test for testing failure/exception for unsupported
+    language(s).
+    """
+    with pytest.raises(ValueError):
+        FortranCTransformation(language='not-supported')
+
+
 @pytest.mark.parametrize('case_sensitive', (False, True))
 @pytest.mark.parametrize('frontend', available_frontends())
 @pytest.mark.parametrize('language', ('c', 'cuda'))
@@ -783,9 +793,7 @@ end subroutine multibody_cond
 
 
 @pytest.mark.parametrize('use_c_ptr', (False, True))
-@pytest.mark.parametrize('frontend', available_frontends(
-    skip=[(OFP, 'Prefix/elemental support not implemented')]
-))
+@pytest.mark.parametrize('frontend', available_frontends())
 def test_transpile_inline_elemental_functions(tmp_path, builder, frontend, use_c_ptr):
     """
     Test correct inlining of elemental functions in C transpilation.
@@ -844,9 +852,7 @@ end subroutine inline_elemental
 
 
 @pytest.mark.parametrize('use_c_ptr', (False, True))
-@pytest.mark.parametrize('frontend', available_frontends(
-    skip=[(OFP, 'Prefix/elemental support not implemented')]
-))
+@pytest.mark.parametrize('frontend', available_frontends())
 def test_transpile_inline_elementals_recursive(tmp_path, builder, frontend, use_c_ptr):
     """
     Test correct inlining of nested elemental functions.
@@ -1088,6 +1094,34 @@ end subroutine add
         assert '#define' not in c_routine
         assert '#endif' not in c_routine
 
+@pytest.mark.parametrize('frontend', available_frontends())
+@pytest.mark.parametrize('codegen', (cgen, cppgen, cudagen))
+def test_transpile_routine_with_interface(tmp_path, frontend, codegen):
+    """
+    Test transpilation of 'INTERFACE's.
+    """
+
+    fcode = """
+subroutine some_routine_with_interf(a, b, result)
+  INTERFACE
+    SUBROUTINE KERNEL(a, b, c)
+      INTEGER, INTENT(INOUT) :: a, b, c
+    END SUBROUTINE KERNEL
+  END INTERFACE
+
+    real, intent(in) :: a, b
+    real, intent(out) :: result
+
+    result = a + b
+end subroutine some_routine_with_interf
+""".strip()
+
+    routine = Subroutine.from_source(fcode, frontend=frontend)
+    f2c = FortranCTransformation()
+    f2c.apply(source=routine, path=tmp_path)
+    c_routine = codegen(routine).lower()
+    assert 'interface' not in c_routine
+    assert 'kernel' not in c_routine
 
 @pytest.mark.parametrize('frontend', available_frontends())
 @pytest.mark.parametrize('f_type', ['integer', 'real'])
@@ -1145,13 +1179,13 @@ end function add
 
 @pytest.mark.parametrize('frontend', available_frontends())
 @pytest.mark.parametrize('codegen', (cgen, cppgen, cudagen))
-def test_transpile_multiconditional(tmp_path, builder, frontend, codegen):
+def test_transpile_multiconditional_simple(tmp_path, builder, frontend, codegen):
     """
     A simple test to verify multiconditionals/select case statements.
     """
 
     fcode = """
-subroutine multi_cond(in, out)
+subroutine multi_cond_simple(in, out)
   implicit none
   integer, intent(in) :: in
   integer, intent(inout) :: out
@@ -1165,7 +1199,7 @@ subroutine multi_cond(in, out)
         out = 100
   end select
 
-end subroutine multi_cond
+end subroutine multi_cond_simple
 """.strip()
 
     # for testing purposes
@@ -1194,7 +1228,7 @@ end subroutine multi_cond
     # compile C version
     libname = f'fc_{routine.name}_{frontend}'
     c_kernel = jit_compile_lib([f2c.wrapperpath, f2c.c_path], path=tmp_path, name=libname, builder=builder)
-    fc_function = c_kernel.multi_cond_fc_mod.multi_cond_fc
+    fc_function = c_kernel.multi_cond_simple_fc_mod.multi_cond_simple_fc
     # test C version
     for i, val in enumerate(test_vals):
         in_var = val
@@ -1203,33 +1237,108 @@ end subroutine multi_cond
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_transpile_multiconditional_range(tmp_path, frontend):
+def test_transpile_multiconditional(tmp_path, builder, frontend):
     """
-    A simple test to verify multiconditionals/select case statements.
+    A test to verify multiconditionals/select case statements.
     """
 
     fcode = """
-subroutine transpile_multi_conditional_range(in, out)
+subroutine multi_cond(in, out)
   implicit none
   integer, intent(in) :: in
   integer, intent(inout) :: out
 
   select case (in)
-    case (1:5)
+    case (:5)
         out = 10
+    case (6, 7, 10:15)
+        out = 15
+    case (8)
+        out = 12
+    case (20:30)
+        out = 20
     case default
         out = 100
   end select
 
-end subroutine transpile_multi_conditional_range
+end subroutine multi_cond
 """.strip()
 
     # for testing purposes
     in_var = 0
-    test_vals = [0, 1, 2, 5, 6]
-    expected_results = [100, 10, 10, 10, 100]
+    # [(<input>, <expected result>), (<input 2>, <expected result 2>), ...]
+    test_results = [(0, 10), (1, 10), (5, 10), (6, 15), (10, 15), (11, 15),
+                    (15, 15), (8, 12), (20, 20), (21, 20), (29, 20), (50, 100)]
     out_var = np.int_([0])
 
+    # compile original Fortran version
+    routine = Subroutine.from_source(fcode, frontend=frontend)
+    filepath = tmp_path/f'{routine.name}_{frontend!s}.f90'
+    function = jit_compile(routine, filepath=filepath, objname=routine.name)
+    # test Fortran version
+    for val in test_results:
+        in_var = val[0]
+        function(in_var, out_var)
+        assert out_var == val[1]
+
+    # apply F2C trafo
+    f2c = FortranCTransformation()
+    f2c.apply(source=routine, path=tmp_path)
+
+    # check whether 'switch' statement is within C code
+    assert 'switch' in cgen(routine)
+
+    # compile C version
+    libname = f'fc_{routine.name}_{frontend}'
+    c_kernel = jit_compile_lib([f2c.wrapperpath, f2c.c_path], path=tmp_path, name=libname, builder=builder)
+    fc_function = c_kernel.multi_cond_fc_mod.multi_cond_fc
+    # test C version
+    for val in test_results:
+        in_var = val[0]
+        fc_function(in_var, out_var)
+        assert out_var == val[1]
+
+
+
+@pytest.mark.parametrize('frontend', available_frontends())
+@pytest.mark.parametrize('dtype', ('integer', 'real',))
+@pytest.mark.parametrize('add_float', (False, True))
+def test_transpile_special_functions(tmp_path, builder, frontend, dtype, add_float):
+    """
+    A simple test to verify multiconditionals/select case statements.
+    """
+    if dtype == 'real':
+        decl_type = f'{dtype}(kind=real64)'
+        kind = '._real64'
+    else:
+        decl_type = dtype
+        kind = ''
+
+    fcode = f"""
+subroutine transpile_special_functions(in, out)
+  use iso_fortran_env, only: real64
+  implicit none
+  {decl_type}, intent(in) :: in
+  {decl_type}, intent(inout) :: out
+  if (mod(in{'+ 2._real64' if add_float else ''}, 2{kind}{'+ 0._real64' if add_float else ''}) .eq. 0) then
+    out = 42{kind}
+  else
+    out = 11{kind}
+  endif
+end subroutine transpile_special_functions
+""".strip()
+
+    def init_var(dtype, val=0):
+        if dtype == 'real':
+            return np.float64([val])
+        return np.int_([val])
+
+    # for testing purposes
+    in_var = init_var(dtype)
+    test_vals = [2, 10, 5, 3]
+    expected_results = [42, 42, 11, 11]
+    out_var = init_var(dtype)
+
     # compile original Fortran version
     routine = Subroutine.from_source(fcode, frontend=frontend)
     filepath = tmp_path/f'{routine.name}_{frontend!s}.f90'
@@ -1243,11 +1352,26 @@ end subroutine transpile_multi_conditional_range
     clean_test(filepath)
 
     # apply F2C trafo
-    # TODO: RangeIndex as case is not yet implemented!
-    #  'NotImplementedError' is raised
     f2c = FortranCTransformation()
-    with pytest.raises(NotImplementedError):
-        f2c.apply(source=routine, path=tmp_path)
+    f2c.apply(source=routine, path=tmp_path)
+
+    # check whether correct modulo was inserted
+    ccode = Path(f2c.c_path).read_text()
+    if dtype == 'integer' and not add_float:
+        assert '%' in ccode
+    if dtype == 'real' or add_float:
+        assert 'fmod' in ccode
+
+    # compile C version
+    libname = f'fc_{routine.name}_{frontend}'
+    c_kernel = jit_compile_lib([f2c.wrapperpath, f2c.c_path], path=tmp_path, name=libname, builder=builder)
+    fc_function = c_kernel.transpile_special_functions_fc_mod.transpile_special_functions_fc
+    # test C version
+    for i, val in enumerate(test_vals):
+        in_var = val
+        fc_function(in_var, out_var)
+        assert int(out_var) == expected_results[i]
+
 
 @pytest.fixture(scope='module', name='horizontal')
 def fixture_horizontal():
@@ -1297,7 +1421,7 @@ def test_scc_cuda_parametrise(tmp_path, here, frontend, config, horizontal, vert
 
     proj = here / '../../tests/sources/projSccCuf/module'
 
-    scheduler = Scheduler(paths=[proj], config=config, seed_routines=['driver'], frontend=frontend)
+    scheduler = Scheduler(paths=[proj], config=config, seed_routines=['driver'], frontend=frontend, xmods=[tmp_path])
 
     dic2p = {'nz': 137}
     cuda_transform = SCCLowLevelParametrise(
@@ -1367,7 +1491,7 @@ def test_scc_cuda_hoist(tmp_path, here, frontend, config, horizontal, vertical,
 
     proj = here / '../../tests/sources/projSccCuf/module'
 
-    scheduler = Scheduler(paths=[proj], config=config, seed_routines=['driver'], frontend=frontend)
+    scheduler = Scheduler(paths=[proj], config=config, seed_routines=['driver'], frontend=frontend, xmods=[tmp_path])
 
     cuda_transform = SCCLowLevelHoist(
         horizontal=horizontal, vertical=vertical, block_dim=blocking,
@@ -1429,3 +1553,108 @@ def test_scc_cuda_hoist(tmp_path, here, frontend, config, horizontal, vertical,
     assert '#include<cuda.h>' in c_elemental_device
     assert '#include<cuda_runtime.h>' in c_elemental_device
     assert '#include"elemental_device_c.h"' in c_elemental_device
+
+
+@pytest.mark.parametrize('frontend', available_frontends())
+@pytest.mark.parametrize('language', ['c', 'cpp'])
+def test_transpile_optional_args(tmp_path, builder, frontend, language):
+    """
+    A simple test to verify multiconditionals/select case statements.
+    """
+
+    fcode = """
+subroutine transpile_optional_args(in, out, out2, opt_flag)
+  implicit none
+  integer, intent(in) :: in
+  integer, intent(inout) :: out
+  integer, intent(out), optional :: out2
+  logical, intent(in), optional :: opt_flag
+
+  out = in
+  if (present(out2)) then
+    out2 = 2*in
+    if (present(opt_flag)) then
+        if (opt_flag) then
+            out = 2* out2
+        else
+            out = 4* out2
+        endif
+    else
+        out = out2
+    endif
+  endif
+  if (.not. present(out2) .and. present(opt_flag)) then
+    if (opt_flag) then
+      out = in + 1
+    else
+      out = in + 2
+    endif
+  endif
+
+end subroutine transpile_optional_args
+""".strip()
+
+    def init_out_vars():
+        return np.int_([0]), np.int_([0])
+
+    builder.clean()
+    # for testing purposes
+    in_var = 10
+
+    # compile and test original Fortran version
+    routine = Subroutine.from_source(fcode, frontend=frontend)
+    filepath = tmp_path/f'{routine.name}_{frontend!s}.f90'
+    function = jit_compile(routine, filepath=filepath, objname=routine.name)
+    out_var, out_var2 = init_out_vars()
+    function(in_var, out_var)
+    assert out_var == 10 and out_var2 == 0
+    out_var, out_var2 = init_out_vars()
+    function(in_var, out_var, out_var2)
+    assert out_var == 20 and out_var2 == 20
+    opt_flag = 1
+    out_var, out_var2 = init_out_vars()
+    function(in_var, out_var, opt_flag=opt_flag)
+    assert out_var == 11 and out_var2 == 0
+    opt_flag = 0
+    out_var, out_var2 = init_out_vars()
+    function(in_var, out_var, opt_flag=opt_flag)
+    assert out_var == 12 and out_var2 == 0
+    opt_flag = 1
+    out_var, out_var2 = init_out_vars()
+    function(in_var, out_var, out_var2, opt_flag)
+    assert out_var == 40 and out_var2 == 20
+    opt_flag = 0
+    out_var, out_var2 = init_out_vars()
+    function(in_var, out_var, out_var2, opt_flag)
+    assert out_var == 80 and out_var2 == 20
+
+    clean_test(filepath)
+
+    # transpile
+    f2c = FortranCTransformation(language=language)
+    f2c.apply(source=routine, path=tmp_path)
+
+    # compile and testC/C++ version
+    libname = f'fc_{routine.name}_{language}_{frontend}'
+    c_kernel = jit_compile_lib([f2c.wrapperpath, f2c.c_path], path=tmp_path, name=libname, builder=builder)
+    fc_function = c_kernel.transpile_optional_args_fc_mod.transpile_optional_args_fc
+    if language != 'c':
+        out_var, out_var2 = init_out_vars()
+        fc_function(in_var, out_var)
+        assert out_var == 10 and out_var2 == 0
+        opt_flag = 1
+        out_var, out_var2 = init_out_vars()
+        fc_function(in_var, out_var, opt_flag=opt_flag)
+        assert out_var == 11 and out_var2 == 0
+        opt_flag = 0
+        out_var, out_var2 = init_out_vars()
+        fc_function(in_var, out_var, opt_flag=opt_flag)
+        assert out_var == 12 and out_var2 == 0
+    opt_flag = 1
+    out_var, out_var2 = init_out_vars()
+    fc_function(in_var, out_var, out_var2, opt_flag)
+    assert out_var == 40 and out_var2 == 20
+    opt_flag = 0
+    out_var, out_var2 = init_out_vars()
+    fc_function(in_var, out_var, out_var2, opt_flag)
+    assert out_var == 80 and out_var2 == 20
